# coding=utf-8
from django.db.models import Q
from django.db.transaction import atomic

from concentrator.smev3 import kinder_conc
from concentrator.smev3.event_service.events import (ACCEPT_DIRECT_EVENT,
                                                     REJECT_DIRECT_EVENT)
from concentrator.smev3.event_service.helpers import (get_event_data,
                                                      push_info_event)
from kinder.core.declaration_status.enum import DSS
from kinder.core.declaration_status.models import DeclarationStatus
from kinder.core.direct.models import (DRS, Direct, DirectStatus,
                                       DirectStatusLog)
from kinder.core.direct.proxy import DirectModel
from kinder.webservice.api.declaration import get_decl_by_client_id


def application_choose_helper(*args, **kwargs):
    """Отправка ответа на выбор ДОО."""
    assert 'message' in kwargs
    message = kwargs.get('message')

    body = message['body'].encode('utf-8')
    request_body = kinder_conc.parseString(body)

    declaration = get_decl_by_client_id(
        request_body.ApplicationChooseRequest.orderId)

    doo_code = request_body.ApplicationChooseRequest.EduOrganizationCode
    answer = request_body.ApplicationChooseRequest.EduOrganizationAnswer

    application_choose(declaration, doo_code, answer)


def application_choose(declaration, doo_code, answer):
    """Отправка события на выбор ДОО."""
    result = application_choose_process(declaration, doo_code, answer)

    if result:
        event = ACCEPT_DIRECT_EVENT
    else:
        event = REJECT_DIRECT_EVENT

    push_info_event(*get_event_data(declaration, event))


@atomic
def application_choose_process(declaration, doo_code, answer):
    """Обработка ответа пользователя."""
    direct = Direct.objects.get(
        ~Q(status__code__in=(
            DRS.ACCEPT,
            DRS.REJECT,
            DRS.REFUSED,
        )),
        group__unit__id=doo_code,
        declaration=declaration,
    )

    old_status = direct.status

    if answer:
        direct_status = DirectStatus.objects.get(code=DRS.ACCEPT)
        decl_status = DeclarationStatus.objects.get(code=DSS.ACCEPTED)
    else:
        direct_status = DirectStatus.objects.get(code=DRS.REJECT)
        decl_status = DeclarationStatus.objects.get(code=DSS.REGISTERED)

    DirectModel.change_status(direct, direct_status)

    DirectStatusLog.objects.register_status(
        direct=direct,
        old_status=old_status,
        status=direct_status,
        comment=u'')

    declaration.change_status(
        to_status=decl_status,
        is_auto=True,
    )

    return answer
